@@ -7,8 +7,9 @@ echo Running ArrayFunc build as a local install.
 echo Compiler messages are redirected to af_compile_results.txt
 
 echo `date` > af_compile_results.txt
-./setup.py install --user 2>> af_compile_results.txt
+echo `date` > af_compile_errors.txt
+python3 -m build  1>> af_compile_results.txt 2>> af_compile_errors.txt
 
-compcount=$( grep "module references __file__" af_compile_results.txt | wc -l )
+compcount=$( grep "$adding 'arrayfunc.*so" af_compile_results.txt | wc -l )
 echo A total of $compcount modules compiled.
 echo Setup complete. Check af_compile_results.txt for errors.
